@@ -11,6 +11,7 @@ import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.ResultSet;
 import java.util.ArrayList;
 
 /**
@@ -83,7 +84,7 @@ public class Deployer
 	private static final int CMD_DATABASE  = 5;
 	private static final int CMD_HOSTNAME  = 6;
 	private static final int CMD_PORT      = 7;
-	private static final int CMD_WINDOWS   = 8;
+	private static final int CMD_CYGWIN    = 8;
 	
 	private final Connection m_connection;
 
@@ -99,7 +100,7 @@ public class Deployer
 		s_commands.add(CMD_DATABASE,  "database");
 		s_commands.add(CMD_HOSTNAME,  "host");
 		s_commands.add(CMD_PORT,      "port");
-		s_commands.add(CMD_WINDOWS,   "windows");
+		s_commands.add(CMD_CYGWIN,    "cygwin");
 	}
 
 	private static final int getCommand(String arg)
@@ -127,7 +128,7 @@ public class Deployer
 		out.println("    [ -database <database> ]    # default is name of current user");
 		out.println("    [ -user <userName>     ]    # default is name of current user");
 		out.println("    [ -password <password> ]    # default is no password");
-		out.println("    [ -windows ]                # If the server is on a Windows machine");
+		out.println("    [ -cygwin ]                 # If the server is on a Cygwin based Windows machine");
 	}
 
 	public static void main(String[] argv)
@@ -139,7 +140,7 @@ public class Deployer
 		String subsystem   = "postgresql";
 		String password    = null;
 		String portNumber  = null;
-		boolean unix       = true;
+		boolean cygwin     = false;
 		int cmd = CMD_UNKNOWN;
 
 		int top = argv.length;
@@ -223,8 +224,8 @@ public class Deployer
 					printUsage();
 					return;
 
-				case CMD_WINDOWS:
-					unix = false;
+				case CMD_CYGWIN:
+					cygwin = true;
 					break;
 
 				default:
@@ -260,6 +261,7 @@ public class Deployer
 					userName,
 					password);
 
+			checkIfConnectedAsSuperuser(c);
 			Deployer deployer = new Deployer(c);
 
 			if(cmd == CMD_UNINSTALL || cmd == CMD_REINSTALL)
@@ -270,7 +272,7 @@ public class Deployer
 			if(cmd == CMD_INSTALL || cmd == CMD_REINSTALL)
 			{
 				deployer.createSQLJSchema();
-				deployer.initJavaHandler(unix);
+				deployer.initJavaHandler(cygwin);
 				deployer.initializeSQLJSchema();
 			}
 			c.close();
@@ -286,11 +288,31 @@ public class Deployer
 		m_connection = c;
 	}
 
+	public static void checkIfConnectedAsSuperuser(Connection conn)
+	throws SQLException
+	{
+		Statement stmt = conn.createStatement();
+		ResultSet rs = stmt.executeQuery("SHOW IS_SUPERUSER");
+
+		try
+		{
+			if(rs.next() && rs.getString(1).equals("on"))
+				return;
+		}
+		finally
+		{
+			rs.close();
+			stmt.close();
+		}
+
+		throw new SQLException(
+			"You must be a superuser to deploy/undeploy pl/Java.");
+	}
+
 	public void dropSQLJSchema()
 	throws SQLException
 	{
 		Statement stmt = m_connection.createStatement();
-		stmt.execute("DROP SCHEMA sqlj CASCADE");
 
 		try
 		{
@@ -300,6 +322,7 @@ public class Deployer
 		{
 			/* ignore */
 		}
+		stmt.execute("DROP SCHEMA sqlj CASCADE");
 		stmt.close();
 	}
 
@@ -308,6 +331,7 @@ public class Deployer
 	{
 		Statement stmt = m_connection.createStatement();
 		stmt.execute("CREATE SCHEMA sqlj");
+		stmt.execute("GRANT USAGE ON SCHEMA sqlj TO public");
 		stmt.close();
 	}
 
@@ -323,6 +347,7 @@ public class Deployer
 			"   jarOrigin   VARCHAR(500) NOT NULL," +
 			"   deploymentDesc INT" +
 		")");
+		stmt.execute("GRANT SELECT ON sqlj.jar_repository TO public");
 
 		stmt.execute(
 			"CREATE TABLE sqlj.jar_entry(" +
@@ -333,6 +358,8 @@ public class Deployer
 			"   UNIQUE(jarId, entryName)" +
 			")");
 
+		stmt.execute("GRANT SELECT ON sqlj.jar_entry TO public");
+
 		stmt.execute(
 			"ALTER TABLE sqlj.jar_repository" +
 			"   ADD FOREIGN KEY (deploymentDesc) REFERENCES sqlj.jar_entry ON DELETE SET NULL");
@@ -346,23 +373,24 @@ public class Deployer
 			"	jarId		INT NOT NULL REFERENCES sqlj.jar_repository ON DELETE CASCADE," +
 			"	PRIMARY KEY(schemaName, ordinal)" +
 			")");
+		stmt.execute("GRANT SELECT ON sqlj.classpath_entry TO public");
 
 		// These are the proposed SQL standard methods.
 		//
 		stmt.execute(
 			"CREATE FUNCTION sqlj.install_jar(VARCHAR, VARCHAR, BOOLEAN) RETURNS void" +
 			"	AS 'org.postgresql.pljava.management.Commands.installJar'" +
-			"	LANGUAGE java");
+			"	LANGUAGE java SECURITY DEFINER");
 
 		stmt.execute(
 			"CREATE FUNCTION sqlj.replace_jar(VARCHAR, VARCHAR, BOOLEAN) RETURNS void" +
 			"	AS 'org.postgresql.pljava.management.Commands.replaceJar'" +
-			"	LANGUAGE java");
+			"	LANGUAGE java SECURITY DEFINER");
 
 		stmt.execute(
 			"CREATE FUNCTION sqlj.remove_jar(VARCHAR, BOOLEAN) RETURNS void" +
 			"	AS 'org.postgresql.pljava.management.Commands.removeJar'" +
-			"	LANGUAGE java");
+			"	LANGUAGE java SECURITY DEFINER");
 
 		// This function is not as proposed. It's more Java'ish. The proposal
 		// using sqlj.alter_jar_path is in my opinion bloated and will not be
@@ -372,27 +400,27 @@ public class Deployer
 		stmt.execute(
 			"CREATE FUNCTION sqlj.set_classpath(VARCHAR, VARCHAR) RETURNS void" +
 			"	AS 'org.postgresql.pljava.management.Commands.setClassPath'" +
-			"	LANGUAGE java");
+			"	LANGUAGE java SECURITY DEFINER");
 
 		stmt.execute(
 			"CREATE FUNCTION sqlj.get_classpath(VARCHAR) RETURNS VARCHAR" +
 			"	AS 'org.postgresql.pljava.management.Commands.getClassPath'" +
-			"	LANGUAGE java STABLE");
+			"	LANGUAGE java STABLE SECURITY DEFINER");
 
 		stmt.close();
 	}
 
-	public void initJavaHandler(boolean unix)
+	public void initJavaHandler(boolean cygwin)
 	throws SQLException
 	{
 		Statement stmt = m_connection.createStatement();
 		stmt.execute(
 			"CREATE FUNCTION sqlj.java_call_handler()" +
 			" RETURNS language_handler" +
-			" AS '" + (unix ? "lib" : "") + "pljava'" +
+			" AS '" + (cygwin ? "" : "lib") + "pljava'" +
 			" LANGUAGE C");
 
-		stmt.execute("CREATE LANGUAGE java HANDLER sqlj.java_call_handler");
+		stmt.execute("CREATE TRUSTED LANGUAGE java HANDLER sqlj.java_call_handler");
 		stmt.close();
 	}
 }
